//
//  DIYCalendarCell.swift
//  FSCalendarSwiftExample
//
//  Created by dingwenchao on 06/11/2016.
//  Copyright © 2016 wenchao. All rights reserved.
//

import Foundation

import UIKit

enum SelectionType : Int {
    case none
    case single
    case leftBorder
    case middle
    case rightBorder
}


class DIYCalendarCell: FSCalendarCell {
    
    weak var selectionLayer: CAShapeLayer!
    
    var selectionType: SelectionType = .none {
        didSet {
            setNeedsLayout()
        }
    }
    
    required init!(coder aDecoder: NSCoder!) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
      
        let color = UIColor.init(named: "蓝色字体颜色")
//        //选中layer
        let selectionLayer = CAShapeLayer()
        selectionLayer.fillColor = color?.cgColor
        selectionLayer.actions = ["hidden": NSNull()]
        self.contentView.layer.insertSublayer(selectionLayer, below: self.titleLabel!.layer)
        self.selectionLayer = selectionLayer
        self.shapeLayer.isHidden = true
        
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()

        self.selectionLayer.frame = self.contentView.bounds
        let color = UIColor.white
        self.selectionLayer.fillColor = color.cgColor
        
        if selectionType == .middle {
            print("middle--middle")
            self.titleLabel.textColor = UIColor.init(named: "蓝色字体颜色")
            
            let color = UIColor.white
            self.selectionLayer.fillColor = color.cgColor
            self.selectionLayer.path = UIBezierPath(rect: self.selectionLayer.bounds).cgPath
        }
        else if selectionType == .leftBorder {
 
            self.titleLabel.textColor = UIColor.white
            
            let color = UIColor.init(named: "蓝色字体颜色")
            self.selectionLayer.fillColor = color?.cgColor
            
            let diameter: CGFloat = min(self.selectionLayer.frame.height, self.selectionLayer.frame.width)
            self.selectionLayer.path = UIBezierPath(ovalIn: CGRect(x: self.bounds.width / 2 - diameter / 2 + 2, y: self.bounds.height / 2 - diameter / 2 - 1, width: diameter - 4, height: diameter - 4)).cgPath
            print("leftBorder--leftBorder")
        }
        else if selectionType == .rightBorder {
 
            print("rightBorder--rightBorder")
            self.titleLabel.textColor = UIColor.white
            let color = UIColor.init(named: "蓝色字体颜色")
            self.selectionLayer.fillColor = color?.cgColor
            let diameter: CGFloat = min(self.selectionLayer.frame.height, self.selectionLayer.frame.width)
            self.selectionLayer.path = UIBezierPath(ovalIn: CGRect(x: self.bounds.width / 2 - diameter / 2 + 2, y: self.bounds.height / 2 - diameter / 2 - 1, width: diameter - 4, height: diameter - 4)).cgPath
        }else if selectionType == .single {
            let color = UIColor.init(named: "蓝色字体颜色")
            self.selectionLayer.fillColor = color?.cgColor
            let diameter: CGFloat = min(self.selectionLayer.frame.height, self.selectionLayer.frame.width)
            self.selectionLayer.path = UIBezierPath(ovalIn: CGRect(x: self.bounds.width / 2 - diameter / 2 + 2, y: self.bounds.height / 2 - diameter / 2 - 1, width: diameter - 4, height: diameter - 4)).cgPath
            print("single--single")
            self.titleLabel.textColor = UIColor.white
        }
    }
    
    override func configureAppearance() {
        super.configureAppearance()
    }
    
}
