//
//  TimeSelectView.swift
//  GeliBusinessPlatform
//
//  Created by 刘俊宏 on 2020/4/23.
//  Copyright © 2020 junye lu. All rights reserved.
//

import UIKit
import SwiftDate

protocol TimeSelectViewDelgate {
    
    func selectMoreDate(beginDate:Date,endginDate:Date)
}

class TimeSelectView: UIView, FSCalendarDataSource, FSCalendarDelegate {
    var delegate:TimeSelectViewDelgate?
    fileprivate weak var calendarC: FSCalendar!
    
    var moreSelection:Bool = false {
        didSet{
            calendarC.allowsMultipleSelection = moreSelection;
            
        }
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.backgroundColor = UIColor.white
        let calendar = FSCalendar(frame: CGRect(x: 0, y: 0, width: frame.size.width, height: frame.size.height))
        calendar.dataSource = self
        calendar.delegate = self
        self.addSubview(calendar)
        self.calendarC = calendar
        self.calendarC.today = nil;
        self.calendarC.appearance.headerMinimumDissolvedAlpha = 0.0;
        calendar.appearance.headerDateFormat = "yyyy年 M 月"
        calendar.allowsMultipleSelection = moreSelection;
        calendar.locale =  Locale.init(identifier: "zh-CN")
        calendar.appearance.useVeryShortWeekdaySymbols = true
        calendar.appearance.headerTitleColor = UIColor.init(named: "标题字颜色")
        calendar.appearance.headerTitleFont = UIFont.systemFont(ofSize: 15)
        calendar.appearance.weekdayTextColor = UIColor.init(named: "灰色字体颜色")
        calendar.appearance.titleDefaultColor = UIColor.init(named: "标题字颜色")
        calendar.appearance.titlePlaceholderColor = UIColor.init(named: "灰色字体颜色")
        calendar.register(DIYCalendarCell.self, forCellReuseIdentifier: "cell")
        
        addChangeBtnUI()
    }
    //切换年
    @objc func changeYear(sender:UIButton) {
        print("切换年\(calendarC.currentPage)")
        
        if sender.tag == 1 {
            let dateA = calendarC.currentPage.addingTimeInterval(TimeInterval(365*60*60*24))
            let nextDate = self.gregorian.date(byAdding: .day, value: 1, to: dateA)!
            
            calendarC.setCurrentPage(nextDate, animated: false)
        }else if sender.tag == 0{
            let dateA = calendarC.currentPage.addingTimeInterval(TimeInterval(-365*60*60*24))
            let nextDate = self.gregorian.date(byAdding: .day, value: 1, to: dateA)!
            calendarC.setCurrentPage(nextDate, animated: false)
            
        }
        
    }
    @objc func changeMoth(sender:UIButton) {
        print("切换月\(calendarC.currentPage)")
        if sender.tag == 1 {
            let dateA = calendarC.currentPage.addingTimeInterval(TimeInterval(30*60*60*24))
            let nextDate = self.gregorian.date(byAdding: .day, value: 1, to: dateA)!
            
            calendarC.setCurrentPage(nextDate, animated: true)
        }else if sender.tag == 0{
            let dateA = calendarC.currentPage.addingTimeInterval(TimeInterval(-30*60*60*24))
            let nextDate = self.gregorian.date(byAdding: .day, value: 1, to: dateA)!
            calendarC.setCurrentPage(nextDate, animated: true)
            
        }
        
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        
    }
    var BeginDate: Date!
    var EndDate: Date!
    var SelectAction: Bool = false
    
    //MARKl:--新增限定时间 ，如果不为nil就是有限定的时间
    var beginDateByWork: Date? = nil
    var endDateByWork: Date? = nil
    
    func setOldDate (beginD: Date , endD:Date) {
        
        let selectArr  = calendarC.selectedDates
        for select in selectArr{
            calendarC.deselect(select)
        }
        calendarC.select(beginD)
        BeginDate = beginD
        
        
        let number = numberOfDaysWithFromDate(fromDate: BeginDate, toDate: endD)!
        EndDate = endD
        for i in 0 ..< number {
            let dateA = endD.addingTimeInterval(TimeInterval(-i*60*60*24))
            calendarC.select(dateA)
        }
        self.configureVisibleCells()
        
    }
    // let date1 = "2018-12-14 08:30:00".toDate()!
    // print("指定日期当天的开始时间：", date1.dateAtStartOf(.day).date)
    //    print("指定日期当天的结束时间：", date1.dateAtEndOf(.day).date)
    
    func clearTheSelect(){
        let selectArr = calendarC.selectedDates
        for select in selectArr {
            calendarC.deselect(select)
        }
        EndDate = nil
        BeginDate = nil
        SelectAction = false
        self.configureVisibleCells()
    }
    //选中某一天进行相关操作
    func calendar(_ calendar: FSCalendar, didSelect date: Date, at monthPosition: FSCalendarMonthPosition) {
        let dateA = self.gregorian.date(byAdding: .day, value: 1, to: date)!
        print("第一次选中\(dateA)")
        if moreSelection {
            if !SelectAction {
                let selectArr  = calendar.selectedDates
                for select in selectArr{
                    calendar.deselect(select)
                }
                if beginDateByWork != nil {
                    let number = numberOfDaysWithFromDate(fromDate: beginDateByWork!, toDate: date)!
                    if number < 0 {
                        let beginDateByWorkT:TimeInterval = beginDateByWork!.timeIntervalSince1970
                        HUD.flash(.label("所选日期不得小于" + dayByDouble(sender: beginDateByWorkT)), delay: 1.2)
                        SelectAction = false
                        if BeginDate != nil && EndDate != nil {
                            setOldDate(beginD: BeginDate, endD: EndDate)
                        }else if BeginDate != nil {
                            let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                            setOldDate(beginD: BeginDate, endD: dateEnd)
                        }
                        calendarC.reloadData()
                        
                    }else{
                        calendarC.select(date)
                        BeginDate = date
                        SelectAction = true
                        EndDate = nil
                        
                        let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                        self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                    }
                    
                }else{
                    calendarC.select(date)
                    BeginDate = date
                    SelectAction = true
                    EndDate = nil
                    let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                    self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                }
                
                
                
            }else{
                //MARK:--日期小于方法
                let number = numberOfDaysWithFromDate(fromDate: BeginDate, toDate: date)!
                if number < 0 {
                    print("小于开始日期，请重新选择")
                    let selectArr  = calendar.selectedDates
                    for select in selectArr{
                        calendar.deselect(select)
                    }
                    if beginDateByWork != nil {
                        let number = numberOfDaysWithFromDate(fromDate: beginDateByWork!, toDate: date)!
                        if number < 0 {
                            let beginDateByWorkT:TimeInterval = beginDateByWork!.timeIntervalSince1970
                            HUD.flash(.label("所选日期不得小于" + dayByDouble(sender: beginDateByWorkT)), delay: 1.2)
                            SelectAction = false
                            if BeginDate != nil && EndDate != nil {
                                setOldDate(beginD: BeginDate, endD: EndDate)
                            }else if BeginDate != nil {
                                let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                                setOldDate(beginD: BeginDate, endD: dateEnd)
                            }
                            calendarC.reloadData()
                            
                        }else{
                            calendarC.select(date)
                            BeginDate = date
                            SelectAction = true
                            EndDate = nil
                            let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                            self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                        }
                        
                    }else{
                        calendarC.select(date)
                        BeginDate = date
                        SelectAction = true
                        EndDate = nil
                        let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                        self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                    }
                    
                }else{
                    SelectAction = false
                    EndDate = date
                    for i in 0 ..< number {
                        let dateA = date.addingTimeInterval(TimeInterval(-i*60*60*24))
                        calendarC.select(dateA)
                    }
                    
                    let dateEnd = EndDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
                    self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                    
                }
            }
        }else{
            //单选
            if endDateByWork != nil {
                if date > endDateByWork! {
                    let endDateByWorkT:TimeInterval = endDateByWork!.timeIntervalSince1970
                    HUD.flash(.label("所选日期不得大于" + dayByDouble(sender: endDateByWorkT)), delay: 1.2)
                    let selectArr  = calendar.selectedDates
                    for select in selectArr{
                        calendar.deselect(select)
                    }
                    calendarC.select(BeginDate)
                    SelectAction = true
                    calendarC.reloadData()
                    
                }else{
                    BeginDate = date
                    EndDate = nil
                    let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1)) //无用处
                    self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                }
            }else{
                BeginDate = date
                EndDate = nil
                let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1)) //无用处
                self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
            }
            
        }
        self.configureVisibleCells()
        
        
    }
    func calendar(_ calendar: FSCalendar, didDeselect date: Date, at monthPosition: FSCalendarMonthPosition) {
        let dateA = self.gregorian.date(byAdding: .day, value: 1, to: date)!
        print("再次点击选择中范围\(dateA)")
        if moreSelection {
            let selectArr = calendar.selectedDates
            for select in selectArr {
                calendar.deselect(select)
            }
            calendarC.select(date)
            SelectAction = true
            BeginDate = date
            EndDate = nil
            let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1))
            self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
        }else{
            //单选
            if endDateByWork != nil {
                if date > endDateByWork! {
                    let endDateByWorkT:TimeInterval = endDateByWork!.timeIntervalSince1970
                    HUD.flash(.label("所选日期不得大于" + dayByDouble(sender: endDateByWorkT)), delay: 1.2)
                    let selectArr  = calendar.selectedDates
                    for select in selectArr{
                        calendar.deselect(select)
                    }
                    calendarC.select(BeginDate)
                    SelectAction = true
                    
                    calendarC.reloadData()
                }else{
                    BeginDate = date
                    EndDate = nil
                    let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1)) //无用处
                    self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
                }
            }else{
                BeginDate = date
                EndDate = nil
                let dateEnd = BeginDate.addingTimeInterval(TimeInterval(60*60*24 - 1)) //无用处
                self.delegate?.selectMoreDate(beginDate: BeginDate, endginDate: dateEnd)
            }
            
            
        }
        self.configureVisibleCells()
    }
    //    //MARK:--最大天数为今日
    //    func maximumDate(for calendar: FSCalendar) -> Date {
    //        if endDateByWork == nil{
    //            return Date().addingTimeInterval(TimeInterval(100*365*60*60*24))//最大为后100年
    //        }
    //        return endDateByWork!
    //    }
    //MARK:--最小天数为今日
    //
    //    func minimumDate(for calendar: FSCalendar) -> Date {
    //        if beginDateByWork == nil{
    //            return Date().addingTimeInterval(TimeInterval(-100*365*60*60*24)) //最小为前100年
    //        }
    //        return beginDateByWork!
    //    }
    //    func calendar(_ calendar: FSCalendar, shouldSelect date: Date, at monthPosition: FSCalendarMonthPosition)   -> Bool {
    //        return monthPosition == .current
    //    }
    //
    //    func calendar(_ calendar: FSCalendar, shouldDeselect date: Date, at monthPosition: FSCalendarMonthPosition) -> Bool {
    //        return monthPosition == .current
    //    }
    
    
    //MARK:--计算日期差
    func numberOfDaysWithFromDate(fromDate:Date,toDate:Date) -> Int? {
        let calendar = Calendar.init(identifier: .gregorian)
        let comp = calendar.dateComponents([.day], from: fromDate, to: toDate)
        return comp.day
        
    }
    
    // MARK:- FSCalendarDataSource
    func calendar(_ calendar: FSCalendar, cellFor date: Date, at position: FSCalendarMonthPosition) -> FSCalendarCell {
        let cell = calendar.dequeueReusableCell(withIdentifier: "cell", for: date, at: position)
        return cell
    }
    
    func calendar(_ calendar: FSCalendar, willDisplay cell: FSCalendarCell, for date: Date, at position: FSCalendarMonthPosition) {
        self.configure(cell: cell, for: date, at: position)
    }
    fileprivate let gregorian = Calendar(identifier: .gregorian)
    private func configureVisibleCells() {
        calendarC.visibleCells().forEach { (cell) in
            let date = calendarC.date(for: cell)
            let position = calendarC.monthPosition(for: cell)
            self.configure(cell: cell, for: date!, at: position)
        }
    }
    private func configure(cell: FSCalendarCell, for date: Date, at position: FSCalendarMonthPosition) {
        let diyCell = (cell as! DIYCalendarCell)
        //        if position == .current {
        var selectionType = SelectionType.none
        
        if calendarC.selectedDates.contains(date) {
            let previousDate = self.gregorian.date(byAdding: .day, value: -1, to: date)!
            let nextDate = self.gregorian.date(byAdding: .day, value: 1, to: date)!
            if calendarC.selectedDates.contains(date) {
                if calendarC.selectedDates.contains(previousDate) && calendarC.selectedDates.contains(nextDate) {
                    selectionType = .middle
                }
                else if calendarC.selectedDates.contains(previousDate) && calendarC.selectedDates.contains(date) {
                    selectionType = .rightBorder
                }
                else if calendarC.selectedDates.contains(nextDate) {
                    selectionType = .leftBorder
                }
                else {
                    selectionType = .single
                }
                diyCell.selectionLayer.isHidden = false
                
            }
        }
        else {
            selectionType = .none
            diyCell.selectionType = selectionType
            diyCell.selectionLayer.isHidden = true
            
        }
        
        if selectionType == .none {
            diyCell.selectionLayer.isHidden = true
            diyCell.selectionType = selectionType
            
            return
        }
        diyCell.selectionLayer.isHidden = false
        diyCell.selectionType = selectionType
        
        //        }
        //    else {
        //            diyCell.selectionType = .none
        //            diyCell.selectionLayer.isHidden = true
        //        }
        
        
        
    }
    
    //MARK:--自定义上下月和上下年UI
    func addChangeBtnUI() {
        //添加上下月和上下年UI
        let previousYbtn = UIButton(type: .custom)
        previousYbtn.backgroundColor = UIColor.white
        self.addSubview(previousYbtn)
        previousYbtn.snp.makeConstraints { (make) in
            make.left.top.equalTo(0)
            make.height.equalTo(30*glscale)
            make.width.equalTo(65*glscale)
        }
        previousYbtn.tag = 0
        previousYbtn.addTarget(self, action: #selector(changeYear(sender:)), for: .touchUpInside)
        
        let previousYImg = UIImageView.init(image: UIImage.init(named: "wangnian"))
        self.addSubview(previousYImg)
        previousYImg.snp.makeConstraints { (make) in
            make.top.equalTo(14)
            make.left.equalTo((15*glscale))
        }
        
        
        let previousMbtn = UIButton(type: .custom)
        previousMbtn.backgroundColor = UIColor.white
        self.addSubview(previousMbtn)
        previousMbtn.snp.makeConstraints { (make) in
            make.top.equalTo(0)
            make.left.equalTo(previousYbtn.snp_right)
            make.height.equalTo(30*glscale)
            make.width.equalTo(65*glscale)
        }
        previousMbtn.tag = 0
        previousMbtn.addTarget(self, action: #selector(changeMoth(sender:)), for: .touchUpInside)
        
        let previousMImg = UIImageView.init(image: UIImage.init(named: "wangyue"))
        self.addSubview(previousMImg)
        previousMImg.snp.makeConstraints { (make) in
            make.top.equalTo(14)
            make.left.equalTo((90*glscale))
        }
        
        
        let nextYbtn = UIButton(type: .custom)
        nextYbtn.backgroundColor = UIColor.white
        self.addSubview(nextYbtn)
        nextYbtn.snp.makeConstraints { (make) in
            make.right.top.equalTo(0)
            make.height.equalTo(30*glscale)
            make.width.equalTo(65*glscale)
        }
        nextYbtn.tag = 1
        nextYbtn.addTarget(self, action: #selector(changeYear(sender:)), for: .touchUpInside)
        
        let nextYImg = UIImageView.init(image: UIImage.init(named: "xiayinian"))
        self.addSubview(nextYImg)
        nextYImg.snp.makeConstraints { (make) in
            make.top.equalTo(14)
            make.right.equalTo((-15*glscale))
        }
        
        let nextMbtn = UIButton(type: .custom)
        nextMbtn.backgroundColor = UIColor.white
        self.addSubview(nextMbtn)
        nextMbtn.snp.makeConstraints { (make) in
            make.top.equalTo(0)
            make.right.equalTo(nextYbtn.snp_left)
            make.height.equalTo(30*glscale)
            make.width.equalTo(65*glscale)
        }
        nextMbtn.tag = 1
        nextMbtn.addTarget(self, action: #selector(changeMoth(sender:)), for: .touchUpInside)
        
        let nextMImg = UIImageView.init(image: UIImage.init(named: "xiageyue"))
        self.addSubview(nextMImg)
        nextMImg.snp.makeConstraints { (make) in
            make.top.equalTo(14)
            make.right.equalTo((-90*glscale))
        }
    }
}
